@@ -8,17 +8,21 @@ package Tickit;
 use strict;
 use warnings;
 
-our $VERSION = '0.17_003';
+BEGIN {
+   our $VERSION = '0.29_001';
+}
 
 use IO::Handle;
 
-use Tickit::Term;
-use Tickit::Window;
-
 use Scalar::Util qw( weaken );
 
-require XSLoader;
-XSLoader::load( __PACKAGE__, $VERSION );
+BEGIN {
+   require XSLoader;
+   XSLoader::load( __PACKAGE__, our $VERSION );
+}
+
+use Tickit::Term;
+use Tickit::Window;
 
 =head1 NAME
 
@@ -30,8 +34,6 @@ C<Tickit> - Terminal Interface Construction KIT
  use Tickit::Widget::Box;
  use Tickit::Widget::Static;
 
- my $tickit = Tickit->new;
-
  my $box = Tickit::Widget::Box->new(
     h_border => 4,
     v_border => 2,
@@ -44,8 +46,7 @@ C<Tickit> - Terminal Interface Construction KIT
     ),
  );
 
- $tickit->set_root_widget( $box );
- $tickit->run;
+ Tickit->new( root => $box )->run;
 
 =head1 DESCRIPTION
 
@@ -61,12 +62,12 @@ via F<libtermkey> (all of these will require a supporting terminal as well).
 It also supports having multiple instances and non-blocking or asynchronous
 control.
 
-At the current version, this is a Perl distribution which contains some XS
-utility code, and depends on a number of other XS modules to provide the
-remaining terminal behaviours. Work is in progress at porting the lower level
-parts into a standalone C library, with the intention being that eventually
-this distribution will become an XS wrapper for that C library. Development
-releases may appear on CPAN that include the C library and XS wrappings of it.
+At the current version, this is a Perl distribution which contains and XS and
+C implementation of the lower levels (L<Tickit::Term> and L<Tickit::Pen>), and
+implements the higher levels (L<Tickit::Window> and L<Tickit::Widget>) in pure
+perl. The XS parts are supported by F<libtickit>, either from the installed
+library, or using a bundled copy compiled at build time. It is intended that
+eventually the Window layer will be rewritten in XS and C instead.
 
 =cut
 
@@ -96,6 +97,11 @@ If defined, overrides locale detection to enable or disable UTF-8 mode. If not
 defined then this will be detected from the locale by using Perl's
 C<${^UTF8LOCALE}> variable.
 
+=item root => Tickit::Widget
+
+If defined, sets the root widget using C<set_root_widget> to the one
+specified.
+
 =back
 
 =cut
@@ -148,16 +154,18 @@ sub new
 
    $term->set_on_key( sub {
       $weakself or return;
-      my ( $term, $type, $str, $key ) = @_;
-      $weakself->on_key( $type, $str, $key );
+      shift; # term
+      $weakself->on_key( @_ );
    } );
 
    $term->set_on_mouse( sub {
       $weakself or return;
-      my ( $term, $ev, $button, $line, $col ) = @_;
-      $weakself->on_mouse( $ev, $button, $line, $col );
+      shift; # term
+      $weakself->on_mouse( @_ );
    } );
 
+   $self->set_root_widget( $args{root} ) if defined $args{root};
+
    return $self;
 }
 
@@ -259,9 +267,9 @@ sub _SIGWINCH
 sub on_key
 {
    my $self = shift;
-   my ( $type, $str, $key ) = @_;
+   my ( $type, $str, $mod ) = @_;
 
-   $self->rootwin->_handle_key( $type, $str, $key ) and return;
+   $self->rootwin->_handle_key( @_ ) and return;
 
    if( exists $self->{key_binds}{$str} ) {
       $self->{key_binds}{$str}->( $self, $str ) and return;
@@ -301,9 +309,8 @@ sub bind_key
 sub on_mouse
 {
    my $self = shift;
-   my ( $ev, $button, $line, $col ) = @_;
 
-   $self->rootwin->_handle_mouse( $ev, $button, $line, $col ) and return;
+   $self->rootwin->_handle_mouse( @_ ) and return;
 }
 
 =head2 $tickit->rootwin
@@ -425,7 +432,7 @@ sub run
 
    $SIG{INT} = $SIG{TERM} = sub { $self->stop };
 
-   $SIG{WINCH} = sub { 
+   $SIG{WINCH} = sub {
       $self->later( sub { $self->_SIGWINCH } )
    };
 
