@@ -1,16 +1,19 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
 
 package Tickit::Rect;
 
 use strict;
 use warnings;
 
-use List::Util qw( min max );
+use Carp;
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.29_001';
+
+# Load the XS code
+require Tickit;
 
 =head1 NAME
 
@@ -42,17 +45,30 @@ simply as a convenient data store containing some useful utility methods.
 Construct a new rectangle of the given geometry, given by C<top>, C<left> and
 either C<lines> and C<cols>, or C<bottom> and C<right>.
 
+=head2 $rect = Tickit::Rect->new( $str )
+
+If given a single string, this will be parsed in the form
+
+ (left,top)..(right,bottom)
+
 =cut
 
 sub new
 {
    my $class = shift;
-   my %args = @_;
+   my %args;
+   if( @_ == 1 ) {
+      @args{qw(left top right bottom)} =
+         $_[0] =~ m/^\((\d+),(\d+)\)..\((\d+),(\d+)\)$/ or croak "Unrecognised Tickit::Rect string '$_[0]'";
+   }
+   else {
+      %args = @_;
+   }
 
    defined $args{lines} or $args{lines} = $args{bottom} - $args{top};
    defined $args{cols}  or $args{cols}  = $args{right}  - $args{left};
 
-   my $self = bless [ @args{qw( top left lines cols )} ], $class;
+   return $class->_new( @args{qw( top left lines cols )} );
 }
 
 =head2 $rect = $existing_rect->intersect( $other_rect )
@@ -62,24 +78,6 @@ return C<undef>.
 
 =cut
 
-sub intersect
-{
-   my $self = shift;
-   my ( $other ) = @_;
-
-   my $top    = max $self->top,    $other->top;
-   my $bottom = min $self->bottom, $other->bottom;
-
-   return undef if $top >= $bottom;
-
-   my $left  = max $self->left,  $other->left;
-   my $right = min $self->right, $other->right;
-
-   return undef if $left >= $right;
-
-   return (ref $self)->new( top => $top, left => $left, bottom => $bottom, right => $right );
-}
-
 =head2 $rect = $existing_rect->translate( $downward, $rightward )
 
 Returns a new rectangle of the same size as the given one, moved down and to
@@ -87,19 +85,6 @@ the right by the given argmuents (which may be negative)
 
 =cut
 
-sub translate
-{
-   my $self = shift;
-   my ( $downward, $rightward ) = @_;
-
-   return (ref $self)->new(
-      top   => $self->top  + $downward,
-      left  => $self->left + $rightward,
-      lines => $self->lines,
-      cols  => $self->cols,
-   );
-}
-
 =head1 ACCESSORS
 
 =cut
@@ -122,19 +107,76 @@ Return the size of the rectangle.
 
 =cut
 
-sub top   { $_[0]->[0] }
-sub left  { $_[0]->[1] }
-sub lines { $_[0]->[2] }
-sub cols  { $_[0]->[3] }
+=head2 @lines = $rect->linerange
 
-sub bottom { $_[0]->[0] + $_[0]->[2] }
-sub right  { $_[0]->[1] + $_[0]->[3] }
+A convenient shortcut to generate the list of lines covered; being
+
+ $rect->top .. $rect->bottom - 1
+
+=cut
+
+sub linerange
+{
+   my $self = shift;
+   return $self->top .. $self->bottom - 1;
+}
+
+=head1 METHODS
+
+=cut
+
+=head2 $bool = $rect->equals( $other )
+
+=head2 $bool = ( $rect == $other )
+
+Returns true if C<$other> represents the same area as C<$rect>.
+
+=cut
+
+use overload '==' => "equals", eq => "equals";
+
+=head2 $bool = $rect->contains( $other )
+
+Returns true if C<$other> is entirely contained within the bounds of C<$rect>.
+
+=cut
+
+=head2 $bool = $rect->intersects( $other )
+
+Returns true if C<$other> and C<$rect> intersect at all, even if they overlap.
+
+=cut
 
 use overload '""' => sub {
    my $self = shift;
    sprintf "Tickit::Rect[(%d,%d)..(%d,%d)]", $self->left, $self->top, $self->right, $self->bottom;
 };
 
+=head2 @r = $rect->add( $other )
+
+Returns a list of the non-overlapping regions covered by either C<$rect> or
+C<$other>.
+
+In the trivial case that the two given rectangles do not touch, the result
+will simply be a list of the two initial rectangles. Otherwise a list of
+newly-constructed rectangles will be returned that covers the same area as
+the original two. This list will contain anywhere between 1 and 3 rectangles.
+
+=cut
+
+=head2 @r = $rect->subtract( $other )
+
+Returns a list of the non-overlapping regions covered by C<$rect> but not by
+C<$other>.
+
+In the trivial case that C<$other> completely covers C<$rect> then the empty
+list is returned. In the trivial case that C<$other> and C<$rect> do not
+intersect then a list containing C<$rect> is returned. Otherwise, a list of
+newly-constructed rectangles will be returned that covers the required area.
+This list will contain anywhere between 1 and 4 rectangles.
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
