@@ -1,18 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2013 -- leonerd@leonerd.org.uk
 
 package Tickit::Term;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.29_001';
 
 # Load the XS code
 require Tickit;
 
+# We export some constants
+use Exporter 'import';
+
 =head1 NAME
 
 C<Tickit::Term> - terminal formatting abstraction
@@ -106,21 +109,21 @@ be passed the C<Tickit::Term> instance, and the new size.
 
 Optional. Event handler function for when a key is pressed. Will be passed the
 C<Tickit::Term> instance, a type string (either C<text> for unmodified Unicode
-or C<key> for special keys or modified Unicode) and a string containing a
-representation of the text or key.
+or C<key> for special keys or modified Unicode), a string containing a
+representation of the text or key, and a modifier bitmask.
 
- $on_key->( $term, $type, $str )
+ $on_key->( $term, $type, $str, $mod )
 
 =item on_mouse => CODE
 
 Optional. Event handler function for when a mouse button is pressed or
 released, or the cursor dragged with a button pressed. Will be passed the
 C<Tickit::Term> instance, a string indicating C<press>, C<drag>, C<release> or
-C<wheel>, the button number or wheel direction, and the 0-based line and
-column index. For C<wheel> events, the direction will be one of C<up> or
-C<down>.
+C<wheel>, the button number or wheel direction, the 0-based line and column
+index, and a modifier bitmask. For C<wheel> events, the direction will be one
+of C<up> or C<down>.
 
- $on_mouse->( $term, $ev, $button_dir, $line, $col )
+ $on_mouse->( $term, $ev, $button_dir, $line, $col, $mod )
 
 =back
 
@@ -133,12 +136,12 @@ sub new
 
    my $self = $class->_new( $ENV{TERM} );
 
-   my $writer = $params{writer};
-   $self->set_output_func( sub { $writer->write( @_ ) } ) if $writer;
-
    $self->set_input_handle ( $params{input_handle}  ) if $params{input_handle};
    $self->set_output_handle( $params{output_handle} ) if $params{output_handle};
 
+   my $writer = $params{writer};
+   $self->set_output_func( sub { $writer->write( @_ ) } ) if $writer;
+
    $self->set_utf8( $params{UTF8} ) if defined $params{UTF8};
 
    $self->set_on_resize( $params{on_resize} ) if $params{on_resize};
@@ -176,20 +179,99 @@ Flushes the output buffer to the terminal
 
 =cut
 
+=head2 $id = $term->bind_event( $ev, $code, $data )
+
+Installs a new event handler to watch for the event specified by C<$ev>,
+invoking the C<$code> reference when it occurs. C<$code> will be invoked with
+the given the terminal, the event name, a C<HASH> reference containing event
+arguments, and the C<$data> value. It returns an ID value that may be used to
+remove the handler by calling C<unbind_event_id>.
+
+ $code->( $term, $ev, $args, $data )
+
+The C<$args> hash will contain keys depending on the event type:
+
+=over 8
+
+=item key
+
+The hash will contain C<type> (a dualvar giving the key event type as an
+integer or string event name, C<text> or C<key>), C<str> (a string containing
+the key event string), and C<mod> (an integer bitmask indicating the modifier
+state).
+
+=item mouse
+
+The hash will contain C<type> (a dualvar giving the mouse event type as an
+integer or string event name, C<press>, C<drag>, C<release> or C<wheel>),
+C<button> (an integer for non-wheel events, or a dualvar for wheel events
+giving the wheel direction as C<up> or C<down>), C<line> and C<col> as
+integers, and C<mod> (an integer bitmask indicating the modifier state).
+
+=item resize
+
+The hash will contain C<lines> and C<cols> as integers.
+
+=back
+
+=cut
+
+=head2 $term->unbind_event_id( $id )
+
+Removes an event handler that returned the given C<$id> value.
+
+=cut
+
 =head2 $term->set_on_resize( $on_resize )
 
 =cut
 
+sub set_on_resize
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   $self->unbind_event_id( delete $self->_event_ids->{resize} ) if exists $self->_event_ids->{resize};
+   $self->_event_ids->{resize} = $self->bind_event( resize => sub {
+      my ( undef, $ev, $args ) = @_;
+      $code->( $self, $args->{lines}, $args->{cols} );
+   } );
+}
+
 =head2 $term->set_on_key( $on_key )
 
 =cut
 
+sub set_on_key
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   $self->unbind_event_id( delete $self->_event_ids->{key} ) if exists $self->_event_ids->{key};
+   $self->_event_ids->{key} = $self->bind_event( key => sub {
+      my ( undef, $ev, $args ) = @_;
+      $code->( $self, $args->{type}, $args->{str} );
+   } );
+}
+
 =head2 $term->set_on_mouse( $on_mouse )
 
 Set a new CODE references to handle events.
 
 =cut
 
+sub set_on_mouse
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   $self->unbind_event_id( delete $self->_event_ids->{mouse} ) if exists $self->_event_ids->{mouse};
+   $self->_event_ids->{mouse} = $self->bind_event( mouse => sub {
+      my ( undef, $ev, $args ) = @_;
+      $code->( $self, $args->{type}, $args->{button}, $args->{line}, $args->{col} );
+   } );
+}
+
 =head2 $term->refresh_size
 
 If a filehandle was supplied to the constructor, fetch the size of the
@@ -223,11 +305,15 @@ Print the given text to the terminal at the current cursor position
 
 =cut
 
-=head2 $term->goto( $line, $col )
+=head2 $success = $term->goto( $line, $col )
 
 Move the cursor to the given position on the screen. If only one parameter is
 defined, does not alter the other. Both C<$line> and C<$col> are 0-based.
 
+Note that not all terminals can support these partial moves. This method
+returns a boolean indicating success; if the terminal could not perform the
+move it will need to be retried using a fully-specified call.
+
 =cut
 
 =head2 $term->move( $downward, $rightward )
@@ -289,9 +375,67 @@ implemented by printing spaces. This removes the need for two cursor jumps.
 
 =cut
 
+=head2 $success = $term->setctl_int( $ctl, $value )
+
+Sets the value of an integer terminal control option. C<$ctl> should be one of
+the following options. They can be specified either as integers, using the
+following named constants, or as strings giving the part following C<TERMCTL_>
+in lower-case.
+
+=over 8
+
+=item TERMCTL_ALTSCREEN
+
+Enables DEC Alternate Screen mode
+
+=item TERMCTL_CURSORVIS
+
+Enables cursor visible mode
+
+=item TERMCTL_CURSORBLINK
+
+Enables cursor blinking mode
+
+=item TERMCTL_CURSORSHAPE
+
+Sets the shape of the cursor. C<$value> should be one of
+C<TERM_CURSORSHAPE_BLOCK>, C<TERM_CURSORSHAPE_UNDER> or
+C<TERM_CURSORSHAPE_LEFT_BAR>.
+
+=item TERMCTL_KEYPAD_APP
+
+Enables keypad application mode
+
+=item TERMCTL_MOUSE
+
+Enables mouse tracking mode
+
+=back
+
+=head2 $success = $term->setctl_str( $ctl, $value )
+
+Sets the value of a string terminal control option. C<$ctrl> should be one of
+the following options. They can be specified either as integers or strings, as
+for C<setctl_int>.
+
+=over 8
+
+=item TERMCTL_ICON_TEXT
+
+=item TERMCTL_TITLE_TEXT
+
+=item TERMCTL_ICONTITLE_TEXT
+
+Sets the terminal window icon text, title, or both.
+
+=back
+
+=cut
+
 =head2 $term->mode_altscreen( $on )
 
-Set or clear the DEC Alternate Screen mode
+Set or clear the DEC Alternate Screen mode. This method is deprecated in
+favour of C<setctl_int>.
 
 =cut
 
@@ -299,12 +443,13 @@ sub mode_altscreen
 {
    my $self = shift;
    my ( $on ) = @_;
-   $self->set_mode_altscreen( $on );
+   $self->setctl_int( altscreen => $on );
 }
 
 =head2 $term->mode_cursorvis( $on )
 
-Set or clear the cursor visible mode
+Set or clear the cursor visible mode. This method is deprecated in favour of
+C<setctl_int>.
 
 =cut
 
@@ -312,12 +457,13 @@ sub mode_cursorvis
 {
    my $self = shift;
    my ( $on ) = @_;
-   $self->set_mode_cursorvis( $on );
+   $self->setctl_int( cursorvis => $on );
 }
 
 =head2 $term->mode_mouse( $on )
 
-Set or clear the mouse tracking mode
+Set or clear the mouse tracking mode. This method is deprecated in favour of
+C<setctl_int>.
 
 =cut
 
@@ -325,7 +471,7 @@ sub mode_mouse
 {
    my $self = shift;
    my ( $on ) = @_;
-   $self->set_mode_mouse( $on );
+   $self->setctl_int( mouse => $on );
 }
 
 =head2 $term->input_push_bytes( $bytes )
