@@ -8,7 +8,7 @@ package Tickit::Test;
 use strict;
 use warnings;
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.29_001';
 
 use Exporter 'import';
 
@@ -208,7 +208,7 @@ sub resize_term
    $tickit->rootwin->resize( $tickit->lines, $tickit->cols );
 }
 
-=head2 presskey( $type, $str )
+=head2 presskey( $type, $str, $mod )
 
 Fire a key event
 
@@ -216,13 +216,12 @@ Fire a key event
 
 sub presskey
 {
-   my ( $type, $str ) = @_;
+   my ( $type, $str, $mod ) = @_;
 
-   # TODO: See if we'll ever need to fake a Term::TermKey::Key event object
-   $tickit->on_key( $type, $str, undef );
+   $tickit->on_key( $type, $str, $mod||0 );
 }
 
-=head2 pressmouse( $ev, $button, $line, $col )
+=head2 pressmouse( $ev, $button, $line, $col, $mod )
 
 Fire a mouse button event
 
@@ -230,9 +229,9 @@ Fire a mouse button event
 
 sub pressmouse
 {
-   my ( $ev, $button, $line, $col ) = @_;
+   my ( $ev, $button, $line, $col, $mod ) = @_;
 
-   $tickit->on_mouse( $ev, $button, $line, $col );
+   $tickit->on_mouse( $ev, $button, $line, $col, $mod||0 );
 }
 
 =head1 TEST FUNCTIONS
@@ -245,7 +244,7 @@ test scripts.
 sub _pen2string
 {
    my $pen = shift;
-   return "{" . join( ",", map { defined $pen->{$_} ? "$_=$pen->{$_}" : "!$_" } sort keys %$pen ) . "}";
+   return "{" . join( ",", map { defined $pen->{$_} ? "$_=" . ($pen->{$_} || 0) : "!$_" } sort keys %$pen ) . "}";
 }
 
 =head2 is_termlog( $log, $name )
@@ -274,7 +273,8 @@ sub is_termlog
       my $want_line = shift @want_log;
       my $got_line  = shift @got_log;
 
-      if( $want_line->[0] eq "chpen_bg" and $got_line->[0] eq "chpen" ) {
+      if( $want_line and $want_line->[0] eq "chpen_bg" and
+          $got_line  and $got_line->[0] eq "chpen" ) {
          $got_line = [ chpen_bg => $got_line->[1]->{bg} ];
       }
 
