@@ -8,11 +8,14 @@ package Tickit::Widget::HBox;
 use strict;
 use warnings;
 use base qw( Tickit::Widget::LinearBox );
+use Tickit::Style;
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.29_001';
 
 use List::Util qw( sum max );
 
+use constant WIDGET_PEN_FROM_STYLE => 1;
+
 =head1 NAME
 
 C<Tickit::Widget::HBox> - distribute child widgets in a horizontal row
@@ -22,9 +25,7 @@ C<Tickit::Widget::HBox> - distribute child widgets in a horizontal row
  use Tickit;
  use Tickit::Widget::HBox;
  use Tickit::Widget::Static;
- 
- my $tickit = Tickit->new;
- 
+
  my $hbox = Tickit::Widget::HBox->new;
 
  foreach my $position (qw( left centre right )) {
@@ -37,14 +38,23 @@ C<Tickit::Widget::HBox> - distribute child widgets in a horizontal row
        expand => 1
     );
  }
- 
- $tickit->set_root_widget( $hbox );
- 
- $tickit->run;
+
+ Tickit->new( root => $hbox )->run;
 
 =head1 DESCRIPTION
 
-This container widget distributes its children in a horizontal row.
+This subclass of L<Tickit::Widget::LinearBox> distributes its children in a
+horizontal row. Its height will be the height of the tallest child, and its
+width will be the sum of the widths of all the children, plus the inter-child
+spacing.
+
+=head1 STYLE
+
+The default style pen is used as the widget pen.
+
+Note that while the widget pen is mutable and changes to it will result in
+immediate redrawing, any changes made will be lost if the widget style is
+changed.
 
 =cut
 
