@@ -1,19 +1,25 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2013 -- leonerd@leonerd.org.uk
 
 package Tickit::Widget::Static;
 
 use strict;
 use warnings;
-use base qw( Tickit::OneLineWidget );
-use Tickit::WidgetRole::Alignable;
+use base qw( Tickit::Widget );
+use Tickit::Style;
 
-our $VERSION = '0.17_003';
+use Tickit::WidgetRole::Alignable name => 'align',  dir => 'h';
+use Tickit::WidgetRole::Alignable name => 'valign', dir => 'v';
 
+our $VERSION = '0.29_001';
+
+use List::Util qw( max );
 use Tickit::Utils qw( textwidth substrwidth );
 
+use constant WIDGET_PEN_FROM_STYLE => 1;
+
 =head1 NAME
 
 C<Tickit::Widget::Static> - a widget displaying static text
@@ -22,22 +28,28 @@ C<Tickit::Widget::Static> - a widget displaying static text
 
  use Tickit;
  use Tickit::Widget::Static;
- 
- my $tickit = Tickit->new;
- 
+
  my $hello = Tickit::Widget::Static->new(
     text   => "Hello, world",
     align  => "centre",
     valign => "middle",
  );
- 
- $tickit->set_root_widget( $hello );
- 
- $tickit->run;
+
+ Tickit->new( root => $hello )->run;
 
 =head1 DESCRIPTION
 
-This class provides a widget which displays a single piece of static text.
+This class provides a widget which displays a single piece of static text. The
+text may contain more than one line, separated by linefeed (C<\n>) characters.
+No other control sequences are allowed in the string.
+
+=head1 STYLE
+
+The default style pen is used as the widget pen.
+
+Note that while the widget pen is mutable and changes to it will result in
+immediate redrawing, any changes made will be lost if the widget style is
+changed.
 
 =cut
 
@@ -66,6 +78,10 @@ Optional. Defaults to C<0.0> if unspecified.
 
 Optional. Defaults to C<0.0> if unspecified.
 
+=item on_click => CODE
+
+Optional. Defaults to C<undef> if unspecified.
+
 =back
 
 For more details see the accessors below.
@@ -81,6 +97,9 @@ sub new
 
    $self->set_text( $args{text} );
    $self->set_align( $args{align} || 0 );
+   $self->set_valign( $args{valign} || 0 );
+
+   $self->set_on_click( $args{on_click} );
 
    return $self;
 }
@@ -89,10 +108,16 @@ sub new
 
 =cut
 
+sub lines
+{
+   my $self = shift;
+   return scalar @{ $self->{lines} };
+}
+
 sub cols
 {
    my $self = shift;
-   return textwidth( $self->{text} );
+   return max map { textwidth $_ } @{ $self->{lines} }
 }
 
 =head2 $text = $static->text
@@ -102,7 +127,7 @@ sub cols
 sub text
 {
    my $self = shift;
-   return $self->{text};
+   return join "\n", @{ $self->{lines} };
 }
 
 =head2 $static->set_text( $text )
@@ -114,7 +139,11 @@ Accessor for C<text> property; the actual text on display in the widget
 sub set_text
 {
    my $self = shift;
-   ( $self->{text} ) = @_;
+   my ( $text ) = @_;
+   my @lines = split m/\n/, $text;
+   # split on empty string returns empty list
+   @lines = ( "" ) if !@lines;
+   $self->{lines} = \@lines;
    $self->resized;
    $self->redraw;
 }
@@ -134,19 +163,54 @@ See also L<Tickit::WidgetRole::Alignable>.
 
 =cut
 
-use constant CLEAR_BEFORE_RENDER => 0;
-
-sub render_line
+sub set_on_click
 {
    my $self = shift;
-   my $window = $self->window;
+   my ( $on_click ) = @_;
 
-   my $text = $self->{text};
-   my ( $left, $textwidth, $right ) = $self->_align_allocation( textwidth( $text ), $window->cols );
+   $self->{on_click} = $on_click;
+}
 
-   $window->erasech( $left, 1 ) if $left;
-   $window->print( substrwidth $text, 0, $textwidth );
-   $window->erasech( $right ) if $right;
+use constant CLEAR_BEFORE_RENDER => 0;
+
+sub render
+{
+   my $self = shift;
+   my %args = @_;
+
+   my $win = $self->window;
+   $win->is_visible or return;
+   my $rect = $args{rect};
+
+   my $cols = $win->cols;
+
+   my ( $above, $lines ) = $self->_valign_allocation( $self->lines, $win->lines );
+
+   $win->goto( $_, 0 ), $win->erasech( $cols ) for $rect->top .. $above - 1;
+
+   foreach my $line ( 0 .. $lines - 1 ) {
+      my $text = $self->{lines}[$line];
+
+      my ( $left, $textwidth, $right ) = $self->_align_allocation( textwidth( $text ), $cols );
+
+      $win->goto( $above + $line, 0 );
+      $win->erasech( $left, 1 ) if $left;
+      $win->print( substrwidth $text, 0, $textwidth );
+      $win->erasech( $right ) if $right;
+   }
+
+   $win->goto( $_, 0 ), $win->erasech( $cols ) for $above + $lines .. $rect->bottom - 1;
+}
+
+sub on_mouse
+{
+   my $self = shift;
+   my ( $ev, $button, $line, $col ) = @_;
+
+   return unless $ev eq "press" and $button == 1;
+   return unless my $on_click = $self->{on_click};
+
+   $on_click->( $self, $line, $col );
 }
 
 =head1 AUTHOR
