@@ -1,14 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2013 -- leonerd@leonerd.org.uk
 
 package Tickit::Window;
 
 use strict;
 use warnings;
+use 5.010; # //
 
-our $VERSION = '0.17_003';
+our $VERSION = '0.29_001';
 
 use Carp;
 
@@ -16,7 +17,10 @@ use Scalar::Util qw( weaken );
 
 use Tickit::Pen;
 use Tickit::Rect;
-use Tickit::Utils qw( textwidth cols2chars );
+use Tickit::RectSet;
+use Tickit::Utils qw( string_countmore );
+
+use constant WEAKEN_CHILDREN => 1;
 
 =head1 NAME
 
@@ -54,10 +58,6 @@ By comparison, any window created by C<make_float> is considered to sit
 other siblings, and any drawing that happens within it overwrites that of its
 non-floating siblings. Any drawing on a non-floating sibling that happens
 within the area of a float is obscured by the contents of the floating window.
-It is recommended that use of floating windows be kept to a minimum, as each
-one imposes a small processing-time overhead in any drawing operations on
-other window. These may be useful for implementing pop-up menus or other
-temporary interactions.
 
 =cut
 
@@ -72,23 +72,75 @@ sub new
    my $self = bless {
       tickit  => $tickit,
       term    => $term,
-      visible => 1,
       top     => 0,
       left    => 0,
       cols    => $cols,
       lines   => $lines,
-      pen     => Tickit::Pen->new,
    }, $class;
+   $self->_init;
 
    weaken( $self->{tickit} );
 
    return $self;
 }
 
+# Shared initialisation by both ->new and ->make_sub
+sub _init
+{
+   my $self = shift;
+   $self->{visible} = 1;
+   $self->{pen}     = Tickit::Pen->new;
+   $self->{damage}  = Tickit::RectSet->new;
+}
+
 =head1 METHODS
 
 =cut
 
+=head2 $win->close
+
+Removes the window from its parent and clears any event handlers set using any
+of the C<set_on_*> methods. Also recursively closes any child windows.
+
+Currently this is an optional method, as child windows are stored as weakrefs,
+so should be destroyed when the last reference to them is dropped. Widgets
+should make sure to call this method anyway, because this will be changed in a
+future version.
+
+=cut
+
+sub DESTROY
+{
+   my $self = shift;
+   $self->close;
+}
+
+sub close
+{
+   my $self = shift;
+
+   $self->set_on_geom_changed( undef );
+   $self->set_on_key( undef );
+   $self->set_on_mouse( undef );
+   $self->set_on_expose( undef );
+   $self->set_on_focus( undef );
+
+   $self->{parent}->_remove( $self ) if $self->{parent};
+   defined and $_->close for @{ $self->{child_windows} };
+}
+
+sub _remove
+{
+   my $self = shift;
+   my ( $child ) = @_;
+
+   my $children = $self->{child_windows};
+   for( my $i = 0; $i < @$children; ) {
+      $i++, next if defined $children->[$i] and $children->[$i] != $child;
+      splice @$children, $i, 1, ();
+   }
+}
+
 =head2 $sub = $win->make_sub( $top, $left, $lines, $cols )
 
 Constructs a new sub-window starting at the given coordinates of this window.
@@ -103,15 +155,31 @@ sub make_sub
 
    my $sub = bless {
       parent  => $self,
-      visible => 1,
-      pen     => Tickit::Pen->new,
    }, ref $self;
+   $sub->_init;
 
    $sub->change_geometry( $top, $left, $lines, $cols );
 
    $self->_reap_dead_children;
    push @{ $self->{child_windows} }, $sub;
-   weaken $self->{child_windows}[-1];
+   weaken $self->{child_windows}[-1] if WEAKEN_CHILDREN;
+
+   return $sub;
+}
+
+=head2 $sub = $win->make_hidden_sub( $top, $left, $lines, $cols )
+
+Constructs a new sub-window like C<make_sub>, but the window starts initially
+hidden. This avoids having to call C<hide> separately afterwards.
+
+=cut
+
+sub make_hidden_sub
+{
+   my $self = shift;
+
+   my $sub = $self->make_sub( @_ );
+   $sub->{visible} = 0;
 
    return $sub;
 }
@@ -130,21 +198,48 @@ sub make_float
 
    my $sub = bless {
       parent  => $self,
-      visible => 1,
-      pen     => Tickit::Pen->new,
-      float   => 1,
    }, ref $self;
+   $sub->_init;
 
    $sub->change_geometry( $top, $left, $lines, $cols );
 
    $self->_reap_dead_children;
    # floats go first
    unshift @{ $self->{child_windows} }, $sub;
-   weaken $self->{child_windows}[0];
+   weaken $self->{child_windows}[0] if WEAKEN_CHILDREN;
 
    return $sub;
 }
 
+=head2 $popup = $win->make_popup( $top, $left, $lines, $cols )
+
+Constructs a new floating popup window starting at the given coordinates
+relative to this window. It will be sized to the given limits.
+
+This window will have the root window as its parent, rather than the window
+the method was called on. Additionally, a popup window will steal all keyboard
+and mouse events that happen, regardless of focus or mouse position. It is
+possible that, if the window has an C<on_mouse> handler, that it may receive
+mouse events from outwide the bounds of the window.
+
+=cut
+
+sub make_popup
+{
+   my $win = shift;
+   my ( $top, $left, $lines, $cols ) = @_;
+
+   while( $win->parent ) {
+      $top  += $win->top;
+      $left += $win->left;
+      $win = $win->parent;
+   }
+
+   my $popup = $win->make_float( $top, $left, $lines, $cols );
+   $popup->{steal_input} = 1;
+   return $popup;
+}
+
 sub _reap_dead_children
 {
    my $self = shift;
@@ -235,7 +330,8 @@ sub hide
 {
    my $self = shift;
    $self->{visible} = 0;
-   if( $self->{float} and my $parent = $self->parent ) {
+
+   if( my $parent = $self->parent ) {
       $parent->_do_expose( $self->rect );
    }
    $self->restore;
@@ -299,7 +395,7 @@ sub change_geometry
    $lines > 0 or croak 'lines zero or negative';
    $cols  > 0 or croak 'cols zero or negative';
 
-   if( !defined $self->{top} or 
+   if( !defined $self->{top} or
        $self->{lines} != $lines or $self->{cols} != $cols or
        $self->{top} != $top or $self->{left} != $left ) {
       $self->{lines} = $lines;
@@ -331,13 +427,12 @@ sub set_on_geom_changed
 Set the callback to invoke whenever a key is pressed while this window, or one
 of its child windows, has the input focus.
 
- $handled = $on_key->( $win, $type, $str, $key )
+ $handled = $on_key->( $win, $type, $str, $mod )
 
 C<$type> will be C<text> for normal unmodified Unicode, or C<key> for special
 keys or modified Unicode. C<$str> will be the UTF-8 string for C<text> events,
 or the textual description of the key as rendered by L<Term::TermKey> for
-C<key> events. C<$key> will be the underlying C<Term::TermKey::Key> event
-structure.
+C<key> events. C<$mod> will be a bitmask of the modifier state.
 
 The invoked code should return a true value if it considers the keypress dealt
 with, or false to pass it up to its parent window.
@@ -371,6 +466,12 @@ sub _handle_key
 
    return 0 unless $self->is_visible;
 
+   $self->_reap_dead_children;
+   my $children = $self->{child_windows};
+   if( $children and @$children and $children->[0]->{steal_input} ) {
+      $children->[0]->_handle_key( @_ ) and return;
+   }
+
    my $focus_child = $self->{focus_child};
    if( $focus_child ) {
       $focus_child->_handle_key( @_ ) and return 1;
@@ -380,7 +481,7 @@ sub _handle_key
       $on_key->( $self, @_ ) and return 1;
    }
 
-   if( my $children = $self->{child_windows} ) {
+   if( $children ) {
       foreach my $child ( @$children ) {
          next unless $child; # weakrefs; may be undef
          next if $focus_child and $child == $focus_child;
@@ -397,12 +498,12 @@ sub _handle_key
 Set the callback to invoke whenever a mouse event is received within the
 window's rectangle.
 
- $handled = $on_mouse->( $win, $ev, $buttons, $line, $col )
+ $handled = $on_mouse->( $win, $ev, $buttons, $line, $col, $mod )
 
 C<$ev> will be C<press>, C<drag> or C<release>. The button number will be in
 C<$button>, though may not be present for C<release> events. C<$line> and
 C<$col> are 0-based. Behaviour of events involving more than one mouse button
-is not well-specified by terminals.
+is not well-specified by terminals. C<$mod> is a bitmask of modifier state.
 
 The invoked code should return a true value if it considers the mouse event
 dealt with, or false to pass it up to its parent window. The mouse event is
@@ -420,7 +521,7 @@ sub set_on_mouse
 sub _handle_mouse
 {
    my $self = shift;
-   my ( $ev, $button, $line, $col ) = @_;
+   my ( $ev, $button, $line, $col, @more ) = @_;
 
    return 0 unless $self->is_visible;
 
@@ -431,15 +532,17 @@ sub _handle_mouse
          my $child_line = $line - $child->top;
          my $child_col  = $col  - $child->left;
 
-         next if $child_line < 0 or $child_line >= $child->lines;
-         next if $child_col  < 0 or $child_col  >= $child->cols;
+         if( !$child->{steal_input} ) {
+            next if $child_line < 0 or $child_line >= $child->lines;
+            next if $child_col  < 0 or $child_col  >= $child->cols;
+         }
 
-         return 1 if $child->_handle_mouse( $ev, $button, $child_line, $child_col );
+         return 1 if $child->_handle_mouse( $ev, $button, $child_line, $child_col, @more );
       }
    }
 
    if( my $on_mouse = $self->{on_mouse} ) {
-      return $on_mouse->( $self, $ev, $button, $line, $col );
+      return $on_mouse->( $self, @_ );
    }
 
    return 0;
@@ -467,68 +570,121 @@ sub _do_expose
    my $self = shift;
    my ( $rect ) = @_;
 
-   undef $self->{expose_pending};
+   # This might be a call from the parent, so flush all pending areas as well
+   $self->{damage}->add( $rect );
+
+   my @rects = $self->{damage}->rects;
+   $self->{damage}->clear;
 
    if( my $on_expose = $self->{on_expose} ) {
-      $on_expose->( $self, $rect );
+      $on_expose->( $self, $_ ) for @rects;
    }
 
    my $children = $self->{child_windows} or return;
 
    foreach my $win ( sort { $a->top <=> $b->top || $a->left <=> $b->left } grep { defined } @$children ) {
-      next unless my $winrect = $rect->intersect( $win->rect );
-      $win->_do_expose( $winrect->translate( -$win->top, -$win->left ) );
+      foreach my $rect ( @rects ) {
+         next unless my $winrect = $rect->intersect( $win->rect );
+         next unless $win->{visible};
+         $win->_do_expose( $winrect->translate( -$win->top, -$win->left ) );
+      }
    }
 }
 
-=head2 $win->expose
+=head2 $win->expose( $rect )
 
-Marks the window as having been exposed, to invoke the C<on_expose> event
-handler on itself, and all its child windows. The window's own handler will be
-invoked first, followed by all the child windows, in screen order (top to
-bottom, then left to right).
+Marks the given region of the window as having been exposed, to invoke the
+C<on_expose> event handler on itself, and all its child windows. The window's
+own handler will be invoked first, followed by all the child windows, in
+screen order (top to bottom, then left to right).
+
+If C<$rect> is not supplied it defaults to exposing the entire window area.
 
 The C<on_expose> event handler isn't invoked immediately; instead, the
 C<Tickit> C<later> method is used to invoke it at the next round of IO event
 handling. Until then, any other window could be exposed. Duplicates are
 suppressed; so if a window and any of its ancestors are both queued for
-expose, the actual handler will only be invoked once.
+expose, the actual handler will only be invoked once per unique region of the
+window.
 
 =cut
 
 sub expose
 {
    my $self = shift;
+   my ( $rect ) = @_;
+   $rect ||= Tickit::Rect->new(
+      top   => 0,
+      left  => 0,
+      lines => $self->lines,
+      cols  => $self->cols,
+   );
 
-   return if $self->_expose_pending;
+   return if $self->_expose_pending( $rect );
 
-   $self->{expose_pending} = 1;
+   $self->{damage}->add( $rect );
 
    $self->tickit->enqueue_redraw( sub {
-      return if $self->parent && $self->parent->_expose_pending;
+      my @rects = $self->{damage}->rects;
+      $self->{damage}->clear;
 
-      undef $self->{expose_pending};
-
-      $self->_do_expose( Tickit::Rect->new(
-         top    => 0,
-         left   => 0,
-         bottom => $self->lines,
-         right  => $self->cols,
-      ) );
+      $self->_expose_pending( $_ ) or $self->_do_expose( $_ ) for @rects;
    } );
 }
 
 sub _expose_pending
 {
    my $self = shift;
-   return $self->{expose_pending} ||
-          ( $self->parent && $self->parent->_expose_pending );
+   my ( $rect ) = @_;
+   return 1 if $self->{damage}->contains( $rect );
+   return 0 unless $self->parent;
+   return $self->parent->_expose_pending( $rect->translate( $self->top, $self->left ) );
 }
 
-=head2 $top  = $win->top
+=head2 $win->set_on_focus( $on_refocus )
+
+Set the callback to invoke whenever the window gains or loses focus.
+
+ $on_refocus->( $win, $has_focus )
+
+Will be passed a boolean value, true if the focus was just gained, false if
+the focus was just lost.
+
+=cut
+
+sub set_on_focus
+{
+   my $self = shift;
+   ( $self->{on_focus} ) = @_;
+}
+
+=head2 $win->set_expose_after_scroll( $expose_after_scroll )
+
+If set to a true value, the C<scrollrect> method will expose the region of
+the window that requires redrawing. If C<scrollrect> was successful, this will
+be just the newly-exposed portion that was scrolled in. If it was unsuccessful
+it will be the entire window region. If set false, no expose will happen, and
+the code calling C<scrollrect> must re-expose as required.
+
+This is a temporary method to handle the transition of behaviours; it may be
+removed in the future and its behaviour implied true always.
+
+=cut
+
+sub set_expose_after_scroll
+{
+   my $self = shift;
+   ( $self->{expose_after_scroll} ) = @_;
+}
+
+=head2 $top = $win->top
+
+=head2 $bottom = $win->bottom
 
 =head2 $left = $win->left
 
+=head2 $right = $win->right
+
 Returns the coordinates of the start of the window, relative to the parent
 window.
 
@@ -540,12 +696,24 @@ sub top
    return $self->{top};
 }
 
+sub bottom
+{
+   my $self = shift;
+   return $self->top + $self->lines;
+}
+
 sub left
 {
    my $self = shift;
    return $self->{left};
 }
 
+sub right
+{
+   my $self = shift;
+   return $self->left + $self->cols;
+}
+
 =head2 $top  = $win->abs_top
 
 =head2 $left = $win->abs_left
@@ -754,20 +922,16 @@ sub _get_span_visibility
          $len = $remains if $len > $remains;
       }
 
-      # Now obscure any floats. Floats always come first so we can stop at the
-      # first non-float for efficiency
       $win->_reap_dead_children;
       foreach my $child ( @{ $win->{child_windows} } ) {
          last if $prev and $child == $prev;
-         last unless $child->{float};
          next unless $child->{visible};
-         next if $child->top > $line or $child->top + $child->lines <= $line;
+         next if $child->top > $line or $child->bottom <= $line;
 
-         my $child_right = $child->left + $child->cols;
-         next if $col >= $child_right;
+         next if $col >= $child->right;
 
          if( $child->left <= $col ) {
-            my $child_cols_hidden = $child_right - $col;
+            my $child_cols_hidden = $child->right - $col;
             if( $vis ) {
                $len = $child_cols_hidden;
                $vis = 0;
@@ -795,7 +959,11 @@ sub _get_span_visibility
 =head2 $win->goto( $line, $col )
 
 Moves the cursor to the given position within the window. Both C<$line> and
-C<$col> are 0-based.
+C<$col> are 0-based. The given position does not have to lie within the bounds
+of the window. Lines above or below the window are never displayed, columns to
+the left or right are clipped as appropriate. The virtual position of the
+cursor is still tracked even if it is not visibly displayed on the actual
+terminal.
 
 =cut
 
@@ -806,10 +974,11 @@ sub goto
 
    $win->{output_line}   = $line;
    $win->{output_column} = $col;
+   $win->{output_needs_goto} = 0;
 
    while( $win ) {
-      $line >= 0 and $line < $win->lines or croak '$line out of bounds';
-      $col  >= 0 and $col  < $win->cols  or croak '$col out of bounds';
+      return if $line < 0 or $line >= $win->lines;
+      return if $col  < 0 or $col  >= $win->cols;
 
       return unless $win->{visible};
 
@@ -817,24 +986,25 @@ sub goto
       $col  += $win->left;
 
       my $parent = $win->parent or last;
-
-      return if $line < 0 or $line >= $parent->lines;
-      return if $col  < 0 or $col  >= $parent->cols;
-
       $win = $parent;
    }
 
    $win->term->goto( $line, $col );
+   $win->_needs_flush;
 }
 
-=head2 $win->print( $text, $pen )
+=head2 $pos = $win->print( $text, $pen )
 
-=head2 $win->print( $text, %attrs )
+=head2 $pos = $win->print( $text, %attrs )
 
 Print the given text to the terminal at the current cursor position using the
 pen of the window, possibly overridden by any extra attributes in the given
 C<Tickit::Pen> instance, or directly in the given hash, if one is provided.
 
+Returns a L<Tickit::StringPos> object giving the total count of string
+printed, including in obscured sections covered by other windows, or clipped
+by window boundaries.
+
 =cut
 
 sub print
@@ -854,42 +1024,61 @@ sub print
    my $line = $self->{output_line};
    my $term = $self->term;
 
-   my $need_goto = 0;
+   my $need_goto = $self->{output_needs_goto};
    my ( $abs_top, $abs_left );
+   my $need_flush = 0;
 
-   while( length $text ) {
-      my ( $vis, $len ) = $self->_get_span_visibility( $line, $self->{output_column} );
+   my $pos = Tickit::StringPos->zero;
 
-      last if !$vis and !defined $len;
+   my $total_len = length $text;
 
-      my $len_ch = cols2chars( $text, $len );
-      my $chunk = substr $text, 0, $len_ch, "";
+   while( $pos->codepoints < $total_len ) {
+      my ( $vis, $cols ) = $self->_get_span_visibility( $line, $self->{output_column} + $pos->columns );
 
-      # truncate $len if it was too high
-      $len = length $chunk;
+      if( !$vis and !defined $cols ) {
+         string_countmore( $text, $pos, undef, $pos->bytes );
+         last;
+      }
+
+      my $prev_cp  = $pos->codepoints;
+      my $prev_col = $pos->columns;
+      defined string_countmore( $text, $pos, Tickit::StringPos->limit_columns( $cols + $pos->columns ), $pos->bytes ) or
+         croak "Encountered non-Unicode text in ->print; bailing out";
+
+      # TODO: This would be more efficient in bytes, but 'use bytes' breaks
+      # the UTF-8ness of the return value
+      my $chunk = substr $text, $prev_cp, $pos->codepoints - $prev_cp;
 
       if( $vis ) {
          if( $need_goto ) {
             $abs_top  //= $self->abs_top;
             $abs_left //= $self->abs_left;
-            $term->goto( $abs_top + $line, $abs_left + $self->{output_column} );
+            $term->goto( $abs_top + $line, $abs_left + $self->{output_column} + $prev_col );
+            $need_goto = 0;
          }
 
          $term->setpen( $pen );
          $term->print( $chunk );
+
+         $need_flush = 1;
       }
       else {
-         # TODO: $term->move( undef, $len )
+         # TODO: $term->move( undef, $pos->columns - $prev_col )
          $need_goto = 1;
       }
-
-      $self->{output_column} += $len;
    }
+
+   $self->{output_column} += $pos->columns;
+
+   $self->root->_needs_flush if $need_flush;
+   $self->{output_needs_goto} = $need_goto;
+
+   return $pos;
 }
 
-=head2 $win->erasech( $count, $moveend, $pen )
+=head2 $pos = $win->erasech( $count, $moveend, $pen )
 
-=head2 $win->erasech( $count, $moveend, %attrs )
+=head2 $pos = $win->erasech( $count, $moveend, %attrs )
 
 Erase C<$count> columns forwards. If C<$moveend> is true, the cursor will be
 placed at the end of the erased region. If defined but false, it will not move
@@ -899,6 +1088,11 @@ option it can implement most efficiently.
 If a C<Tickit::Pen> or pen attributes are provided, they are used to override
 the background colour for the erased region.
 
+Returns a L<Tickit::StringPos> object giving the total count of string
+printed, including in obscured sections covered by other windows, or clipped
+by window boundaries. Only the C<columns> field will be valid; the others will
+be C<-1>.
+
 =cut
 
 sub erasech
@@ -919,9 +1113,11 @@ sub erasech
    my $line = $self->{output_line};
    my $term = $self->term;
 
-   my $need_goto = 0;
+   my $need_goto = $self->{output_needs_goto};
    my ( $abs_top, $abs_left );
+   my $need_flush = 0;
 
+   my $orig_count = $count;
    while( $count ) {
       my ( $vis, $len ) = $self->_get_span_visibility( $line, $self->{output_column} );
 
@@ -934,10 +1130,13 @@ sub erasech
             $abs_top  //= $self->abs_top;
             $abs_left //= $self->abs_left;
             $term->goto( $abs_top + $line, $abs_left + $self->{output_column} );
+            $need_goto = 0;
          }
 
          $term->setpen( $pen );
          $term->erasech( $len, $moveend );
+
+         $need_flush = 1;
       }
       else {
          $need_goto = 1;
@@ -946,6 +1145,43 @@ sub erasech
       $self->{output_column} += $len;
       $count -= $len;
    }
+
+   $self->root->_needs_flush if $need_flush;
+   $self->{output_needs_goto} = $need_goto;
+
+   return if !defined wantarray;
+   return Tickit::StringPos->limit_columns( $orig_count );
+}
+
+=head2 $win->clearrect( $rect )
+
+=head2 $win->clearrect( $rect, $pen )
+
+=head2 $win->clearrect( $rect, %attrs )
+
+Erase the content of the window within the given L<Tickit::Rect>. If a
+C<Tickit::Pen> or pen attributes are provided, they are used to override the
+background colour for the erased region.
+
+=cut
+
+sub clearrect
+{
+   my $self = shift;
+   my $rect = shift;
+
+   my $pen = ( @_ == 1 ) ? shift->clone : Tickit::Pen->new( @_ );
+
+   if( $rect->top == 0 and $rect->left == 0 and
+       $rect->bottom == $self->lines and $rect->right == $self->cols ) {
+      $self->clear;
+      return;
+   }
+
+   foreach my $line ( $rect->linerange ) {
+      $self->goto( $line, $rect->left );
+      $self->erasech( $rect->cols, undef, $pen );
+   }
 }
 
 =head2 $success = $win->scrollrect( $top, $left, $lines, $cols, $downward, $rightward )
@@ -969,15 +1205,31 @@ undefined if this method returns successful. The terminal pen, in particular
 the background colour, may be modified by this method even if it fails to
 scroll the terminal (and returns false).
 
+If the C<expose_after_scroll> behavior is enabled, then this method will
+enqueue all of the required expose requests before returning, so in this case
+the return value is not interesting.
+
 =cut
 
-sub scrollrect
+sub _scrollrect_inner
 {
-   my $win = shift;
-   my ( $top, $left, $lines, $cols, $downward, $rightward, @args ) = @_;
+   my $self = shift;
+   my ( $rect, $downward, $rightward, @args ) = @_;
+
+   if( abs($downward) >= $rect->lines or abs($rightward) >= $rect->cols ) {
+      $self->expose( $rect ) if $self->{expose_after_scroll};
+      return 1;
+   }
 
    my $pen = ( @args == 1 ) ? $args[0]->clone : Tickit::Pen->new( @args );
 
+   my $top  = $rect->top;
+   my $left = $rect->left;
+
+   my $lines = $rect->lines;
+   my $cols  = $rect->cols;
+
+   my $win = $self;
    while( $win ) {
       $top  >= 0 and $top  < $win->lines or croak '$top out of bounds';
       $left >= 0 and $left < $win->cols  or croak '$left out of bounds';
@@ -996,19 +1248,109 @@ sub scrollrect
       $win = $parent;
    }
 
-   # Check there are no floating windows in the way
-   foreach my $line ( $top .. $top + $lines - 1 ) {
-      my ( $vis, $len ) = $win->_get_span_visibility( $line, $left );
-      return 0 unless $vis;
-      return 0 unless $len >= $cols;
-   }
-
    my $term = $win->term;
 
    $term->setpen( bg => $pen->getattr( 'bg' ) );
 
-   return $term->scrollrect( $top, $left, $lines, $cols,
-      $downward, $rightward );
+   unless( $term->scrollrect( $top, $left, $lines, $cols, $downward, $rightward ) ) {
+      $self->expose( $rect ) if $self->{expose_after_scroll};
+      return 0;
+   }
+
+   if( $self->{expose_after_scroll} ) {
+      if( $downward > 0 ) {
+         # "scroll down" means lines moved upward, so the bottom needs redrawing
+         $self->expose( Tickit::Rect->new(
+               top  => $rect->bottom - $downward, lines => $downward,
+               left => $rect->left,               cols  => $rect->cols,
+         ) );
+      }
+      elsif( $downward < 0 ) {
+         # "scroll up" means lines moved downward, so top needs redrawing
+         $self->expose( Tickit::Rect->new(
+               top  => $rect->top,  lines => -$downward,
+               left => $rect->left, cols  => $rect->cols,
+         ) );
+      }
+
+      if( $rightward > 0 ) {
+         # "scroll right" means columns moved leftward, so the right edge needs redrawing
+         $self->expose( Tickit::Rect->new(
+               top  => $rect->top,                lines => $rect->lines,
+               left => $rect->right - $rightward, cols  => $rightward,
+         ) );
+      }
+      elsif( $rightward < 0 ) {
+         # "scroll left" means lines moved rightward, so left edge needs redrawing
+         $self->expose( Tickit::Rect->new(
+               top  => $rect->top,  lines => $rect->lines,
+               left => $rect->left, cols  => -$rightward,
+         ) );
+      }
+   }
+   else {
+      $self->_needs_flush;
+   }
+
+   return 1;
+}
+
+# TODO: This ought probably to take a Tickit::Rect instead of 4 ints
+sub scrollrect
+{
+   my $self = shift;
+   my ( $top, $left, $lines, $cols, $downward, $rightward, @args ) = @_;
+
+   my $rect = Tickit::Rect->new(
+      top   => $top,
+      left  => $left,
+      lines => $lines,
+      cols  => $cols,
+   );
+
+   my $visible = Tickit::RectSet->new;
+   $visible->add( $rect );
+
+   my $right = $left + $cols;
+
+   foreach my $line ( $top .. $top + $lines - 1 ) {
+      my $col = $left;
+      while( $col < $right ) {
+         my ( $vis, $len ) = $self->_get_span_visibility( $line, $col );
+         $col += $len and next if $vis;
+         my $until = defined $len ? $col + $len : $right;
+
+         return 0 unless $self->{expose_after_scroll};
+
+         $visible->subtract( Tickit::Rect->new(
+            top    => $line,
+            bottom => $line+1,
+            left   => $col,
+            right  => $until,
+         ) );
+
+         $col = $until;
+      }
+   }
+
+   my @rects = $self->{damage}->rects;
+   $self->{damage}->clear;
+   foreach my $r ( @rects ) {
+      $self->{damage}->add( $r ), next if $r->bottom < $top or $r->top > $top + $lines or
+                                          $r->right < $left or $r->left > $left + $cols;
+      my $inside = $r->intersect( $rect );
+      my @outside = $r->subtract( $rect );
+
+      $self->{damage}->add( $_ ) for @outside;
+      $self->{damage}->add( $inside->translate( -$downward, -$rightward ) ) if $inside;
+   }
+
+   my $ret = 1;
+   foreach my $r ( $visible->rects ) {
+      $self->_scrollrect_inner( $r, $downward, $rightward, @args ) or $ret = 0;
+   }
+
+   return $ret;
 }
 
 =head2 $success = $win->scroll( $downward, $rightward )
@@ -1043,6 +1385,8 @@ sub focus
    $self->{focus_line} = $line;
    $self->{focus_col}  = $col;
 
+   $self->{on_focus}->( $self, 1 ) if $self->{on_focus};
+
    $self->_requeue_focus
 }
 
@@ -1055,7 +1399,8 @@ sub _requeue_focus
       $self->{focus_child}->_lose_focus;
    }
 
-   weaken( $self->{focus_child} = $focuswin );
+   $self->{focus_child} = $focuswin;
+   weaken $self->{focus_child} if WEAKEN_CHILDREN;
 
    if( my $parent = $self->parent ) {
       $parent->_requeue_focus( $self );
@@ -1083,6 +1428,20 @@ sub _lose_focus
 
    undef $self->{focus_line};
    undef $self->{focus_col};
+
+   $self->{on_focus}->( $self, 0 ) if $self->{on_focus};
+}
+
+=head2 $focused = $win->is_focused
+
+Returns true if this window currently has the input focus
+
+=cut
+
+sub is_focused
+{
+   my $self = shift;
+   return defined $self->{focus_line};
 }
 
 =head2 $win->restore
@@ -1113,7 +1472,7 @@ sub restore
       $root->_gain_focus;
    }
 
-   $term->flush;
+   $root->_needs_flush;
 }
 
 =head2 $win->clearline( $line )
@@ -1153,9 +1512,24 @@ sub clear
       my $term = $self->term;
       $term->setpen( $self->get_effective_pen );
       $term->clear;
+
+      $self->_needs_flush;
    }
 }
 
+sub _needs_flush
+{
+   my $self = shift;
+   return if $self->{flush_queued};
+
+   $self->tickit->later( sub {
+      $self->term->flush;
+      undef $self->{flush_queued};
+   } );
+
+   $self->{flush_queued}++;
+}
+
 use overload
    '""' => sub {
       my $self = shift;
@@ -1167,7 +1541,7 @@ use overload
       my $self = shift;
       return $self;
    },
-   fallback => 1; 
+   fallback => 1;
 
 =head1 AUTHOR
 
